@@ -24,7 +24,6 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <string.h>
-#include <strings.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/types.h>
@@ -134,9 +133,9 @@ add_id(struct idcache **ic, char *name, unsigned long int id, int *width)
 	else
 		*ic = nc;
 	if (w <= 0)
-		w = strlen(nc->name);
-	*width = *width < w ? w : *width;
+		w = nc->name ? strlen(nc->name) : 0;
 
+	*width = *width < w ? w : *width;
 	return;
 }
 
@@ -179,6 +178,7 @@ readlink_to_namei(struct namei *nm, const char *path)
 {
 	char sym[PATH_MAX];
 	ssize_t sz;
+	int isrel = 0;
 
 	sz = readlink(path, sym, sizeof(sym));
 	if (sz < 1)
@@ -186,20 +186,26 @@ readlink_to_namei(struct namei *nm, const char *path)
 	if (*sym != '/') {
 		char *p = strrchr(path, '/');
 
-		nm->relstart = p ? p - path : 0;
-		if (nm->relstart)
+		if (p) {
+			isrel = 1;
+			nm->relstart = p ? p - path : 0;
 			sz += nm->relstart + 1;
+		}
 	}
 	nm->abslink = xmalloc(sz + 1);
 
-	if (*sym != '/' && nm->relstart) {
+	if (*sym != '/' && isrel) {
 		/* create the absolute path from the relative symlink */
 		memcpy(nm->abslink, path, nm->relstart);
 		*(nm->abslink + nm->relstart) = '/';
 		nm->relstart++;
 		memcpy(nm->abslink + nm->relstart, sym, sz - nm->relstart);
 	} else
+		/* - absolute link (foo -> /path/bar)
+		 * - or link without any subdir (foo -> bar)
+		 */
 		memcpy(nm->abslink, sym, sz);
+
 	nm->abslink[sz] = '\0';
 }
 
